;-------------------------------------------------------------------------------
;
; Title:            ds30 Loader for PIC18
;
; File description: UART functionality
;
; Copyright:        Copyright 2011-2020, DigSol Sweden AB
;
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;    This file is part of ds30 Loader.
;
;    ds30 Loader is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation.
;
;    ds30 Loader is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with ds30 Loader. If not, see <http://www.gnu.org/licenses/>.
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
; UARTs
;-------------------------------------------------------------------------------
        #ifdef USE_UART1
            #ifdef USE_UART2
                error "Both uarts are specified"
            #endif

            #define PIRRCIF_            PIR1
            #define RCIF_               RCIF
            #define TRMT_               TRMT1
            #define BRG16_              BRG161

            #ifndef TXSTA1
                #define TXSTA_          TXSTA
                #define RCSTA_          RCSTA
                #define SPBRG_          SPBRG
                #define TXREG_          TXREG
                #define RCREG_          RCREG
            #else
                #define TXSTA_          TXSTA1
                #define RCSTA_          RCSTA1
                #define SPBRG_          SPBRG1
                #define TXREG_          TXREG1
                #define RCREG_          RCREG1
            #endif

            #ifdef BAUDCON
                #define BAUDCON_        BAUDCON
            #else
                #define BAUDCON_        BAUDCON1
            #endif

            #ifdef SPBRGH
                #define SPBRGH_         SPBRGH
            #else
                #define SPBRGH_         SPBRGH1
            #endif
        #endif

        #ifdef USE_UART2
            #ifdef USE_UART1
                error "Both uarts are specified"
            #endif
            #ifndef HAS_UART2
                error "UART2 specified for a device that only has uart1"
            #endif


            #define PIRRCIF_            PIR3
            #define RCIF_               RC2IF
            #define TRMT_               TRMT2
            #define BRG16_              BRG162

               #define  TXSTA_          TXSTA2
               #define  RCSTA_          RCSTA2
               #define  SPBRG_          SPBRG2
            #define     SPBRGH_         SPBRGH2
               #define  TXREG_          TXREG2
               #define  RCREG_          RCREG2
            #define     BAUDCON_        BAUDCON2
        #endif

        ;
        #ifdef USE_TXENABLE
            #define    TXE_DELAY_CNT    ( (TXE_DELAY * OSCF) / (4000000 * 3) )
        #endif


;-------------------------------------------------------------------------------
; Range check
;-------------------------------------------------------------------------------
        #ifndef USE_BRG16
            #if  UBRG > 255
                error spbrg_value_ is out of range
            #endif
        #else
            #if  UBRG > 65535
                error spbrg_value_ is out of range
            #endif
        #endif

        #if  UBRG == 0
            error spbrg_value_ might be out of range
        #endif


;-------------------------------------------------------------------------------
; CommInit()
;-------------------------------------------------------------------------------
CommInit:
        banksel         TXSTA_
        #ifdef USE_BRG16
            bsf         BRG16_
            movlw       (UBRG>>8)
            movwf       SPBRGH_
        #endif

        movlw           0x24              ;enable transmit and
        movwf           TXSTA_              ; high speed mode
        movlw           (UBRG&0xff)
        movwf           SPBRG_
        movlw           0x90              ;enable serial port
        movwf           RCSTA_              ; and receive

        #ifdef USE_ABAUD
            ; Activate auto baud rate detection
            bsf         BAUDCON_, ABDEN
            ; Wait for auto baudrate detection to finish with timeout control
            movlw       BLSTART
            movwf       cnt1
abd_rpt2:   movlw       DELBASE
            movwf       cnt2
abd_rpt3:   clrf        cnt3
abd_rptc:   clrwdt
            btfss       BAUDCON_, ABDEN
            bra         abd_ok
abd_notrcv: decfsz      cnt3
            bra         abd_rptc
            decfsz      cnt2
            bra         abd_rpt3
            decfsz      cnt1
            bra         abd_rpt2
            ; abd timed out if we get here
            bra         exit
            ; abd finished
abd_ok:     movf        RCREG_, w           ;clear rcif
            SendL       OK

        #endif

        ; Make tx enable pin output and set to 0
        #ifdef USE_TXENABLE
            bcf         TRISR_TXE, TRISB_TXE
            bcf         LATR_TXE, LATB_TXE
        #endif

        movlb           0x0
        return


;-------------------------------------------------------------------------------
; CommExit()
;-------------------------------------------------------------------------------
CommExit:
        banksel         RCSTA_
        clrf            RCSTA_              ;reset receive status and control register
        clrf            TXSTA_              ;reset transmit status and control register
        #ifdef USE_BRG16
            clrf        BAUDCON_
            clrf        SPBRGH_
        #endif

        movlb           0x0
        return


;-------------------------------------------------------------------------------
; Receive()
;-------------------------------------------------------------------------------
Receive:movlw           BLDELAY
RcvIni: movwf           cnt1
rpt2:   movlw           DELBASE
        movwf           cnt2
rpt3:   clrf            cnt3
rptc:
        #ifdef KICK_WD
            clrwdt
        #endif
        btfss           RCIF_
        bra             notrcv
        banksel         RCREG_
        movf            RCREG_, w           ;return read data in W
        movlb           0x0
        addwf           crc, f              ;compute checksum
        return
notrcv: decfsz          cnt3
        bra             rptc
        decfsz          cnt2
        bra             rpt3
        decfsz          cnt1
        bra             rpt2
        ; Receive timed out if we get here
        bra             exit


;-------------------------------------------------------------------------------
; Send()
;-------------------------------------------------------------------------------
Send:   ; Enable tx
        banksel         TXSTA_
        #ifdef USE_TXENABLE
            bcf         RCSTA_, CREN        ;disable serial receiver
            movwf       cnt2
            bsf         LATR_TXE, LATB_TXE
            movlw       TXE_DELAY_CNT
            movwf       cnt1
txe_del_lo: decfsz      cnt1
            bra         txe_del_lo
            movf        cnt2, w
        #else
            ; Wait for transmit shift register to get empty
txwait:     btfss       TRMT_
            bra         txwait
        #endif
        ;Send byte
        movwf           TXREG_
        ; Disable tx
        #ifdef USE_TXENABLE
            ; Wait for transmit shift register to get empty
txwait:     btfss       TRMT_
            bra         txwait
            bcf         LATR_TXE, LATB_TXE
            bsf         RCSTA_, CREN        ;enable serial receiver
        #endif
        ; Send complete
        movlb           0x0
        return


;-------------------------------------------------------------------------------
; End of file
;-------------------------------------------------------------------------------
